@@ -1,28 +1,12 @@
-/*
- * Copyright (C) 2017 ~ 2018 Deepin Technology Co., Ltd.
- *
- * Author:     sbw <sbw@sbw.so>
- *
- * Maintainer: sbw <sbw@sbw.so>
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #ifndef PACKAGESMANAGER_H
 #define PACKAGESMANAGER_H
 
 #include "utils/result.h"
+#include "model/dependgraph.h"
 
 #include <QApt/Backend>
 #include <QApt/DebFile>
@@ -425,19 +409,6 @@ private:
      */
     void addPackage(int validPkgCount, QString packagePath, QByteArray packageMd5Sum);
 
-    /**
-     * @brief swapPackages   对列表中第一个包位置交换
-     * @param packagePath    包的路径
-     */
-    void swapPackages(const QString &packagePath);
-
-    /**
-     * @brief swappedPackageIndex 获取当前包要插入的位置
-     * @param packagePath     包的路径
-     * @return
-     */
-    int swappedPackageIndex(const QString &packagePath);
-
     /**
      * @brief getAllDepends 获取安装包所有依赖
      * @param depends       依赖列表
@@ -602,11 +573,13 @@ private:
 
     QList<QVector<QString>> m_orDepends; //存储或依赖关系
 
-    QList<QVector<QString>> m_checkedOrDepends; //存储还未检测的或依赖关系
+    QList<QVector<QString>> m_unCheckedOrDepends; //存储还未检测的或依赖关系
 
     QMap<QString, PackageDependsStatus> m_checkedOrDependsStatus; //存储检测完成的或依赖包及其依赖状态
 
     QMap<QString, DependencyInfo> m_dependsInfo; //所有依赖的信息
+
+    DependGraph m_dependGraph; //依赖关系图计算器
 };
 
 #endif  // PACKAGESMANAGER_H
