@@ -1,20 +1,6 @@
-/*
-* Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
-*
-* This program is free software: you can redistribute it and/or modify
-* it under the terms of the GNU General Public License as published by
-* the Free Software Foundation, either version 3 of the License, or
-* any later version.
-*
-* This program is distributed in the hope that it will be useful,
-* but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-* GNU General Public License for more details.
-*
-* You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
-
+// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "debinstaller.h"
 #include "model/deblistmodel.h"
@@ -107,13 +93,17 @@ void DebInstaller::initTitleBar()
 {
     //title bar settings
     DTitlebar *tb = titlebar();
-
+    QAction *settingAction(new QAction(tr("Settings"), this));
+    DMenu *menu = new DMenu;
+    menu->addAction(settingAction);
+    tb->setMenu(menu);
     if (tb) {
         tb->setIcon(QIcon::fromTheme("deepin-deb-installer"));
         tb->setTitle("");
         tb->setAutoFillBackground(true);
         tb->setDisableFlags(Qt::CustomizeWindowHint);
     }
+    connect(settingAction, &QAction::triggered, this, &DebInstaller::slotSettingDialogVisiable);
 }
 
 void DebInstaller::initConnections()
@@ -181,7 +171,7 @@ void DebInstaller::disableCloseAndExit()
 {
     titlebar()->setDisableFlags(Qt::WindowCloseButtonHint);             //设置标题栏中的关闭按钮不可用
     DTitlebar *tbar = this->titlebar();
-    if (tbar) {
+    if(tbar){
         tbar->setQuitMenuDisabled(true);
     }
 
@@ -194,7 +184,7 @@ void DebInstaller::enableCloseAndExit()
                                 ~Qt::WindowCloseButtonHint);
 
     DTitlebar *tbar = this->titlebar();
-    if (tbar) {
+    if(tbar){
         tbar->setQuitMenuDisabled(false);
     }
 }
@@ -319,7 +309,7 @@ void DebInstaller::slotShowPkgRemovedMessage(QString packageName)
 
 void DebInstaller::slotShowUninstallConfirmPage()
 {
-    m_fileListModel->setWorkerStatus(DebListModel::WorkerUnInstall);                        //刷新当前安装器的工作状态
+    m_fileListModel->setWorkerStatus( DebListModel::WorkerUnInstall);                       //刷新当前安装器的工作状态
 
     this->setAcceptDrops(false);                                                                //卸载页面不允许添加/拖入包
 
@@ -503,7 +493,7 @@ void DebInstaller::slotSetEnableButton(bool bButtonEnabled)
     if (m_packageAppending)
         return;
     //Set button enabled after installation canceled
-    if (2 == m_dragflag) { //单包安装按钮的启用与禁用
+    if (2 == m_dragflag ) {//单包安装按钮的启用与禁用
         SingleInstallPage *singlePage = qobject_cast<SingleInstallPage *>(m_lastPage);
         if (singlePage)
             singlePage->setEnableButton(bButtonEnabled);
